# Roger Palay copyright 2016-02-14
# Saline, MI 48176
#
ci_2known <- function (
  sigma_one, n_one, x_one,
  sigma_two, n_two, x_two,
  cl=0.95)
{
    # try to avoid some common errors
    if( (cl <=0) | (cl>=1) )
      {return("Confidence interval must be strictly between 0.0 and 1")
      }
    if( (sigma_one <= 0)  | (sigma_two <= 0) )
      {return("Population standard deviation must be positive")}
    if( (as.integer(n_one) != n_one ) |
        (as.integer(n_two) != n_two ) )
      {return("Sample size must be a whole number")}

    pe <- x_one - x_two
    sd <- sqrt(sigma_one^2/n_one + sigma_two^2/n_two)
    alpha <- 1-cl
    z_half_alpha <- qnorm(alpha/2,lower.tail=FALSE)
    moe <- z_half_alpha*sd
    low_end <- pe-moe
    high_end <- pe+moe
    result <- c(low_end, high_end, pe, 
                sd, moe, alpha/2,
                z_half_alpha)
    names(result)<-c("CI Low","CI High","Pnt. Est.",
                    "St. Error", "M of E",
                    "alpha/2","z-score")
    return( result )
}