# Roger Palay copyright 2016-01-31
# Saline, MI 48176
#
ci_stddev <- function(  n=30, s=0, cl=0.95)
{
  # try to avoid some common errors
  if( cl <=0 | cl>=1)
  {return("Confidence interval must be strictly between 0.0 and 1")
  }
  if( s <= 0 )
  {return("Sample standard deviation must be positive")}
  if( n <= 1 )
  {return("Sample size needs to be more than 1")}
  if( as.integer(n) != n )
  {return("Sample size must be a whole number")}
  # to get here we have some "reasonable" values
  alpha = (1-cl)/2 # area at each side
  lval <- qchisq(alpha,n-1)
  rval <- qchisq(alpha,n-1,lower.tail=FALSE)
  low_end <- sqrt((n-1)*s*s/rval)
  high_end <- sqrt((n-1)*s*s/lval)
  result <- c(low_end, high_end, n-1, lval, rval)
  names(result)<-c("CI Low","CI High","deg. of freedom","left chisq", "right chisq")
  return( result )
}